#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <algorithm>

using namespace std;

// Generates random integer from range 0, 1, ..., Max - 1 uniformly
long long random_integer(long long Max) {
  long long ans = Max;
  while (ans >= Max) {
    ans = 0;
    for (int i = 0; (1LL << i) < Max; ++i) if (rand() % 2) {
      ans |= (1LL << i);
    }
  }
  return ans;
}

// Generates random integer from range Min, Min + 1, ..., Max
long long random_integer(long long Min, long long Max) {
  return Min + random_integer(Max - Min + 1);
}

#define MAX_VALUE 10000000000000000LL
int TESTCASE_ID;
void gen_testcase(int group_id, int minN, int maxN, long long maxM, double ratio) {
  TESTCASE_ID++;
  fprintf(stderr, "Group: %d Testcase: %d\n", group_id, TESTCASE_ID);
  char filename[100];
  sprintf(filename, "data/bobek.%d%c", group_id, 'a' + TESTCASE_ID - 1);
  FILE *f = fopen(filename, "w");
  int N = random_integer(minN, maxN);
  long long M = random_integer(1, maxM);
  fprintf(f, "%d %lld\n", N, M);
  for (int j = 0; j < N; ++j) {
    if (j) fprintf(f, " ");
    double value_limit = max(2.0, min((double)MAX_VALUE, M * 2 * ratio / N));
    long long cost = random_integer(1, value_limit);
    fprintf(f, "%lld", cost);
  }
  fprintf(f, "\n");
  fclose(f);
}

void gen_testcase(int id, int maxN, long long maxM) {
  fprintf(stderr, "-------------------------\n");
  TESTCASE_ID = 0;
  for (int i = 0; i < 3; ++i) { // uniformly at random, N in top 1/4
    gen_testcase(id, maxN * 3 / 4, maxN, maxM, 2.0);
  }
  // generate N uniformly
  gen_testcase(id, 1, maxN, maxM, 2.0);
  // small budget
  gen_testcase(id, maxN * 3 / 4, maxN, min(maxM, 200LL), 2.0);
  gen_testcase(id, maxN * 3 / 4, maxN, maxM, 0.1); // tiny costs
  gen_testcase(id, maxN * 3 / 4, maxN, maxM, 1000.0); // huge costs
}

long long power(int m, int e) {
  long long ans = 1;
  while (e--) ans *= m;
  return ans;
}

int main(int argc, char *argv[]) {
  FILE *f;

  srand(347856124);

  f = fopen ("data/bobek.1", "w");
  fprintf (f, "1 1\n1\n");
  fclose (f);
  gen_testcase(2, 10, power(10, 6));
  gen_testcase(3, 20, power(10, 15));
  gen_testcase(4, 20, power(10, 15));
  gen_testcase(5, 40, power(10, 6));
  gen_testcase(6, 40, power(10, 6));
  gen_testcase(7, 40, power(10, 6));
  gen_testcase(8, 40, power(10, 15));
  gen_testcase(9, 40, power(10, 15));
  gen_testcase(10, 40, power(10, 15));
  return 0;
}
